/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.logging.logmanager;

import java.security.AccessController;
import java.security.PrivilegedAction;
import org.jboss.logging.LoggerPlugin;
import org.jboss.logging.LoggerPluginInstance;
import org.jboss.logging.MDCProvider;
import org.jboss.logging.NDCProvider;
import org.jboss.logging.MDCSupport;
import org.jboss.logging.NDCSupport;
import org.jboss.logmanager.Logger;

/**
 *
 */
public class LoggerPluginImpl implements LoggerPlugin, MDCSupport, NDCSupport {
    private final MDCProvider mdcProvider;
    private final NDCProvider ndcProvider;
    private static final Logger.AttachmentKey<LoggerPluginInstance> KEY = new Logger.AttachmentKey<LoggerPluginInstance>();

    public LoggerPluginImpl() {
        mdcProvider = new MDCProviderImpl();
        ndcProvider = new NDCProviderImpl();
    }

    public LoggerPluginInstance getInstance(String name, String resourceBundleName) {
        Logger logger = resourceBundleName == null ? Logger.getLogger(name) : Logger.getLogger(name, resourceBundleName);
        LoggerPluginInstance lpi = logger.getAttachment(KEY);
        if (lpi == null) {
            final SecurityManager sm = System.getSecurityManager();
            lpi = new LoggerPluginInstanceImpl(name, resourceBundleName, logger, this);
            if (sm != null) {
                return AccessController.doPrivileged(new AttachAction(logger, lpi));
            } else {
                return doAttach(logger, lpi);
            }
        }
        return lpi;
    }

    public MDCProvider getMDCProvider() {
        return mdcProvider;
    }

    public NDCProvider getNDCProvider() {
        return ndcProvider;
    }

    private static final class AttachAction implements PrivilegedAction<LoggerPluginInstance> {
        private final Logger logger;
        private final LoggerPluginInstance attachment;

        private AttachAction(final Logger logger, final LoggerPluginInstance attachment) {
            this.logger = logger;
            this.attachment = attachment;
        }

        public LoggerPluginInstance run() {
            final Logger logger = this.logger;
            final LoggerPluginInstance attachment = this.attachment;
            return doAttach(logger, attachment);
        }
    }

    private static LoggerPluginInstance doAttach(final Logger logger, final LoggerPluginInstance attachment) {
        LoggerPluginInstance appeared = logger.attachIfAbsent(KEY, attachment);
        if (appeared != null) {
            return appeared;
        }
        return attachment;
    }
}
